# -*- coding: utf-8 -*-
#
# Copyright © 2015 Xavier Lamien.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# __author__ = 'Xavier Lamien <laxathom@fedoraproject.org>'

import os
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend


def encipher(data, secret, signature=None):
    """
    Encipher a given data.

    :param data: The data to encipher
    :type data: str
    :param secret: Secret key required to encipher the data
    :type secret: str
    :param signature: The name of data's receiver
    :type signature: str
    :return: The data Enciphered
    :rtype: Union[_AEADEncryptionContext, _AEADCipherContext, _CipherContext]
    """
    # Generate a random 96-bit IV.
    iv = os.urandom(12)

    encryptor = Cipher(
        algorithms.AES(secret), modes.GCM(iv),
        backend=default_backend()
    ).encryptor()

    encryptor.authenticate_additional_data(signature)

    cipher_data = encryptor.update(data) + encryptor.finalize()

    return iv, cipher_data, encryptor.tag


def decipher(data, iv, secret, signature, tag):
    """
    Deciphers a given data.

    :param data: The data to decipher
    :type data: str
    :param iv:
    :type iv: str
    :param secret: The secret key required to decipher the data
    :type secret: str
    :param signature: THe signature to verify the enciphered data
    :return: The data deciphered
    :rtype: list of _AEADEncryptionContext, _AEADCipherContext, _CipherContext
    """
    # Construct a Cipher object, with the key, iv, and additionally the
    # GCM tag used for authenticating the message.
    decryptor = Cipher(
        algorithms.AES(secret), modes.GCM(iv, tag),
        backend=default_backend()
    ).decryptor()

    # We put associated_data back in or the tag will fail to verify
    # when we finalize the decryptor.
    decryptor.authenticate_additional_data(signature)

    return decryptor.update(data) + decryptor.finalize()
